from flask import render_template, redirect, url_for, flash, request, g
from datetime import datetime
from flask_login import current_user, login_required
from app import db
from app.reports.forms import OrderForm, MonthlyRestaurantReportForm, EditOrderForm
from app.auth.permissions import driver_permission
from app.models import User, Order, Restaurant, PaymentType
from app.reports import bp


@bp.route('/new_order', methods=['GET', 'POST'])
@login_required
@driver_permission.require()
def new_order():
    """
    The new order form. Should be the default view for drivers.
    """
    form = OrderForm()
    if not g.identity.allow_admin:
        form.date.render_kw = {'disabled': 'disabled'}
    if form.validate_on_submit():

        order = Order(
            date=form.date.data,
            driver=current_user.username,
            receipt=form.receipt.data,
            customer_address=form.customer_addr.data,
            driver_paid_amount=form.driver_paid.data,
            total_restaurant_amount=form.restaurant_total.data,
            tips_amount=form.tips.data,
            additional_pickup=form.additional_pickup.data,
            restaurant_id=Restaurant.query.filter_by(name=form.restaurant.data).first().id,
            payment_type_id=PaymentType.query.filter_by(name=form.customer_payment_type.data).first().id,
            user_id=User.query.filter_by(username=current_user.username).first().id
        )
        user = User.query.filter_by(username=current_user.username).first()
        if form.last_order.data:
            user.set_inactive()
            user.check_active_drivers()
        else:
            user.set_active()
        db.session.add(order)
        db.session.commit()
        flash('Order submitted')
        return redirect(url_for('main.index'))



    elif request.method == 'GET':
        form.date.data = datetime.utcnow()
        return render_template('index.html', title='New Order', form=form)


@bp.route('/order_management', methods=['GET', 'POST'])
@login_required
@driver_permission.require()
def order_management():
    from datetime import timedelta
    today = datetime.today()
    today_at_zero = today + timedelta(days=-1)
    today_at_midnight = today + timedelta(days=1)
    orders = Order.query.filter(Order.date.between(today_at_zero, today_at_midnight)).filter(Order.driver == current_user.username)
    return render_template('reports/order_management.html', title='Admin Panel - Order Management', orders=[o for o in orders])


@bp.route('/order_management/edit_order/<order_id>', methods=['GET', 'POST'])
@login_required
@driver_permission.require()
def edit_order(order_id):
    order = Order.query.filter_by(id=order_id).first()
    form = EditOrderForm()

    daily_driver = False
    weekly_restaurant = False

    if form.validate_on_submit():
        order.date = form.date.data
        order.restaurant = Restaurant.query.filter_by(name=form.restaurant.data).first().id
        order.receipt = form.receipt.data
        order.customer_address = form.customer_address.data
        order.driver_paid_amount = form.driver_paid_amount.data
        order.total_restaurant_amount = form.total_restaurant_amount.data
        order.tips_amount = form.tips.data
        order.payment_type_id = PaymentType.query.filter_by(name=form.customer_payment_type.data).first().id
        order.additional_pickup = form.additional_pickup.data

        flash("Order {} updated".format(order.id))
        # Was missing db.session.commit()
        db.session.commit()

        driver = User.query.filter_by(id=current_user.id).first()

        if form.last_order.data:
            driver.set_active()
        else:
            driver.set_inactive()
            daily_driver, weekly_restaurant = driver.check_active_drivers()

        if daily_driver:
            flash("Running Daily Driver Reports")
        if weekly_restaurant:
            flash("Running Weekly Restaurant Reports")

        db.session.commit()
        return render_template('index.html', title='Edit Order', form=form)

    elif request.method == 'GET':
        form.date.data = order.date
        form.restaurant.data = order.restaurant()
        form.receipt.data = order.receipt
        form.customer_address.data = order.customer_address
        form.driver_paid_amount.data = order.driver_paid_amount
        form.total_restaurant_amount.data = order.total_restaurant_amount
        form.tips.data = order.tips_amount
        form.customer_payment_type.data = order.customer_payment_type()
        form.additional_pickup.data = order.additional_pickup

    return render_template('index.html', title='Edit Order', form=form)


@bp.route('/generate_report', methods=['GET', 'POST'])
@driver_permission.require()
def generate_report():
    """
    renders the template that provides options for which report to run, showing the correct form for which report
    """
    if current_user.is_authenticated:
        return redirect(url_for('main.index'))
    form = MonthlyRestaurantReportForm()
    if form.validate_on_submit():
        user = User(username=form.username.data, email=form.email.data)
        user.set_password(form.password.data)
        db.session.add(user)
        db.session.commit()
        flash('Congratulations, you are now a registered user!')
        return redirect(url_for('auth.login'))
    return render_template('auth/register.html', title='Register', form=form)



